import os
import uuid

from cpl_core.configuration import ConfigurationABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_core.mailing import EMailClientABC, EMailClientSettings
from cpl_core.utils import CredentialManager
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from flask import jsonify, Response
from flask import request
from requests_oauthlib import OAuth2Session

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.api import Api
from bot_api.configuration.discord_authentication_settings import (
    DiscordAuthenticationSettings,
)
from bot_api.logging.api_logger import ApiLogger
from bot_api.model.auth_user_dto import AuthUserDTO
from bot_api.route.route import Route
from bot_data.model.auth_role_enum import AuthRoleEnum

# Disable SSL requirement
os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "1"


class AuthDiscordController:
    BasePath = "/api/auth/discord"

    def __init__(
        self,
        auth_settings: DiscordAuthenticationSettings,
        config: ConfigurationABC,
        env: ApplicationEnvironmentABC,
        logger: ApiLogger,
        bot: DiscordBotServiceABC,
        t: TranslatePipe,
        api: Api,
        mail_settings: EMailClientSettings,
        mailer: EMailClientABC,
        auth_service: AuthServiceABC,
    ):
        self._auth_settings = auth_settings
        self._config = config
        self._env = env
        self._logger = logger
        self._bot = bot
        self._t = t
        self._api = api
        self._mail_settings = mail_settings
        self._mailer = mailer
        self._auth_service = auth_service

    def _get_user_from_discord_response(self) -> dict:
        discord = OAuth2Session(
            self._bot.user.id,
            redirect_uri=self._auth_settings.redirect_url,
            state=request.args.get("state"),
            scope=self._auth_settings.scope.to_list(),
        )
        token = discord.fetch_token(
            self._auth_settings.token_url,
            client_secret=CredentialManager.decrypt(self._auth_settings.client_secret),
            authorization_response=request.url,
        )
        discord = OAuth2Session(self._bot.user.id, token=token)
        return discord.get("https://discordapp.com/api" + "/users/@me").json()

    @Route.get(f"{BasePath}/get-url")
    async def get_url(self):
        oauth = OAuth2Session(
            self._bot.user.id,
            redirect_uri=self._auth_settings.redirect_url,
            scope=self._auth_settings.scope.to_list(),
        )
        login_url, state = oauth.authorization_url(self._auth_settings.auth_url)
        return jsonify({"loginUrl": login_url})

    @Route.get(f"{BasePath}/login")
    async def discord_login(self) -> Response:
        response = self._get_user_from_discord_response()
        dto = AuthUserDTO(
            0,
            response["username"],
            response["discriminator"],
            response["email"],
            str(uuid.uuid4()),
            None,
            AuthRoleEnum.normal,
        )

        result = await self._auth_service.login_discord_async(dto, response["id"])
        return jsonify(result.to_dict())
