from datetime import datetime
from typing import Union

import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_discord.events import OnMemberRemoveABC
from cpl_translation import TranslatePipe

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.event_checks import EventChecks
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.configuration.base_server_settings import BaseServerSettings


class BaseOnMemberRemoveEvent(OnMemberRemoveABC):
    def __init__(
        self,
        logger: LoggerABC,
        base_helper: BaseHelperABC,
        db: DatabaseContextABC,
        messenger: MessageServiceABC,
        users: UserRepositoryABC,
        servers: ServerRepositoryABC,
        user_joins: UserJoinedServerRepositoryABC,
        translate: TranslatePipe,
    ):
        OnMemberRemoveABC.__init__(self)

        self._logger = logger
        self._base_helper = base_helper
        self._db = db
        self._messenger = messenger
        self._users = users
        self._servers = servers
        self._user_joins = user_joins
        self._t = translate

    async def _remove_user(self, member: Union[discord.User, discord.Member]):
        self._logger.debug(__name__, f"Remove user {member}")
        settings: BaseServerSettings = self._base_helper.get_config(member.guild.id)
        await self._messenger.send_dm_message(self._t.transform("modules.base.goodbye_message"), member)

        try:
            server = self._servers.get_server_by_discord_id(member.guild.id)

            user = self._users.find_user_by_discord_id_and_server_id(member.id, server.id)
            if user is None:
                self._logger.error(__name__, f"Cannot find user {member}")
                return

            join = self._user_joins.get_active_user_joined_server_by_user_id(user.id)
            join.leaved_on = datetime.now()
            self._user_joins.update_user_joined_server(join)
            self._db.save_changes()
        except Exception as e:
            self._logger.error(__name__, f"Cannot remove user {member.id}", e)

    @EventChecks.check_is_ready()
    async def on_member_remove(self, member: discord.Member):
        self._logger.debug(__name__, f"Module {type(self)} started")
        await self._remove_user(member)
