import discord
from cpl_core.database.context import DatabaseContextABC
from cpl_discord.container import Guild
from cpl_discord.service import DiscordBotServiceABC
from discord import Permissions, Colour

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.data_seeder_abc import DataSeederABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.level import Level
from bot_data.model.server import Server
from bot_data.service.level_repository_service import LevelRepositoryService
from modules.level.configuration.default_level_settings import DefaultLevelSettings
from modules.level.service.level_service import LevelService


class LevelSeeder(DataSeederABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        levels: DefaultLevelSettings,
        level_repo: LevelRepositoryService,
        servers: ServerRepositoryABC,
        level: LevelService,
        db: DatabaseContextABC,
        bot: DiscordBotServiceABC,
    ):
        DataSeederABC.__init__(self)

        self._logger = logger
        self._levels = level_repo
        self._servers = servers
        self._level = level
        self._db = db
        self._bot = bot

        self._level_header = levels.level_header
        self._default_levels = levels.levels.order_by_descending(lambda l: l.min_xp)

    async def _create_level(self, level: Level, guild: Guild, server: Server):
        level.server = server
        try:
            if guild.roles.where(lambda r: r.name == level.name).first_or_default() is None:
                await guild.create_role(
                    name=level.name,
                    colour=Colour(int(level.color, 16)),
                    hoist=False,
                    mentionable=True,
                    permissions=Permissions(level.permissions),
                )
                self._logger.debug(__name__, f"Created role {level.name}")

            levels = self._levels.find_levels_by_server_id(server.id)
            if levels is None or levels.where(lambda l: l.name == level.name).first_or_default() is None:
                self._levels.add_level(level)
                self._logger.debug(__name__, f"Saved level {level.name}")
                self._db.save_changes()
        except discord.errors.Forbidden as e:
            self._logger.error(__name__, f"Creating level failed", e)
            level.permissions = 0
            self._levels.update_level(level)
        except Exception as e:
            self._logger.error(__name__, f"Creating level failed", e)

    async def seed(self):
        # create levels
        for guild in self._bot.guilds:
            created_default = False
            if guild.roles.where(lambda r: r.name == self._level_header).first_or_default() is None:
                await guild.create_role(name=self._level_header)

            server = self._servers.find_server_by_discord_id(guild.id)
            if server is None:
                continue

            levels = self._levels.find_levels_by_server_id(server.id)
            if levels is not None and levels.count() > 0:
                # create levels from db
                for level in levels:
                    await self._create_level(level, guild, server)

                self._logger.debug(__name__, f"Seeded levels")
            else:
                # create default levels
                for level in self._default_levels:
                    created_default = True
                    await self._create_level(level, guild, server)

                self._logger.debug(__name__, f"Seeded default levels")

            if created_default:
                continue

            levels = levels.order_by_descending(lambda l: l.min_xp)
            position_above_levels = guild.roles.where(lambda r: r.name == self._level_header).single().position
            for role in guild.roles.order_by_descending(lambda r: r.position):
                if levels.where(lambda l: l.name == role.name).count() == 0:
                    continue

                new_position = position_above_levels - (
                    levels.index_of(levels.where(lambda l: l.name == role.name).single()) + 1
                )
                if new_position <= 0:
                    new_position = 1
                try:
                    self._logger.debug(
                        __name__,
                        f"Moved {role.name} from {role.position} to {new_position}",
                    )
                    await role.edit(position=new_position)
                except Exception as e:
                    self._logger.error(__name__, f"Cannot change position of {role.name}", e)

            for m in guild.members:
                await self._level.check_level(m)

        self._logger.debug(__name__, f"Checked role order")
