import { Injectable } from "@angular/core";
import { MenuItem } from "primeng/api";
import { BehaviorSubject } from "rxjs";
import { AuthRoles } from "../../models/auth/auth-roles.enum";
import { AuthService } from "../auth/auth.service";
import { LangChangeEvent, TranslateService } from "@ngx-translate/core";
import { NavigationEnd, Router } from "@angular/router";
import { ThemeService } from "../theme/theme.service";
import { Server } from "../../models/data/server.model";
import { UserDTO } from "../../models/auth/auth-user.dto";

@Injectable({
  providedIn: "root"
})
export class SidebarService {

  isSidebarOpen: boolean = true;
  menuItems$ = new BehaviorSubject<MenuItem[]>(new Array<MenuItem>());
  server$ = new BehaviorSubject<Server | null>(null);

  dashboard: MenuItem = {};
  serverDashboard: MenuItem = {};
  serverProfile: MenuItem = {};
  serverMembers: MenuItem = {};
  serverAutoRoles: MenuItem = {};
  serverLevels: MenuItem = {};
  serverMenu: MenuItem = {};
  adminConfig: MenuItem = {};
  adminUsers: MenuItem = {};
  adminMenu: MenuItem = {};

  constructor(
    private themeService: ThemeService,
    private authService: AuthService,
    private translateService: TranslateService,
    private router: Router
  ) {
    this.themeService.isSidebarOpen$.subscribe(value => {
      this.isSidebarOpen = value;
      this.setMenu();
    });

    this.translateService.onLangChange.subscribe(_ => {
      this.setMenu(true);
    });

    this.router.events.subscribe(event => {
      if (!(event instanceof NavigationEnd)) {
        return;
      }
      if (!event.url.startsWith("/server/") && this.server$.value) {
        this.setServer(null);
      }
    });
  }

  setServer(server: Server | null) {
    if (server?.id != this.server$.value?.id) {
      this.server$.next(server);
      if (server) {
        this.setMenu(true);
      } else {
        this.setMenu(false);
      }
    }
  }

  async buildMenu(user: UserDTO | null, hasPermission: boolean) {
    this.dashboard = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.dashboard") : "",
      icon: "pi pi-th-large",
      routerLink: "dashboard"
    };
    this.serverDashboard = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.dashboard") : "",
      icon: "pi pi-th-large",
      routerLink: `server/${this.server$.value?.id}`
    };
    this.serverProfile = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.profile") : "",
      icon: "pi pi-id-card",
      routerLink: `server/${this.server$.value?.id}/members/${user?.id}`
    };
    this.serverMembers = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.members") : "",
      icon: "pi pi-users",
      visible: true,
      routerLink: `server/${this.server$.value?.id}/members`
    };

    this.serverAutoRoles = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.auto_roles") : "",
      icon: "pi pi-sitemap",
      visible: true,
      routerLink: `server/${this.server$.value?.id}/auto-roles`
    };

    this.serverLevels = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.server.levels") : "",
      icon: "pi pi-book",
      visible: true,
      routerLink: `server/${this.server$.value?.id}/levels`
    };

    this.serverMenu = {
      label: this.isSidebarOpen ? this.server$.value?.name : "",
      icon: "pi pi-server",
      visible: false,
      expanded: true,
      items: [this.serverDashboard, this.serverProfile, this.serverMembers, this.serverAutoRoles, this.serverLevels]
    };
    this.adminConfig = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.config") : "",
      icon: "pi pi-cog",
      routerLink: "/admin/settings"
    };
    this.adminUsers = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.auth_user_list") : "",
      icon: "pi pi-user-edit",
      routerLink: "/admin/users"
    };
    this.adminMenu = {
      label: this.isSidebarOpen ? this.translateService.instant("sidebar.administration") : "",
      icon: "pi pi-cog",
      visible: hasPermission,
      expanded: true,
      items: [this.adminConfig, this.adminUsers]
    };
  }

  setMenu(build: boolean = false) {
    this.authService.hasUserPermission(AuthRoles.Admin).then(async hasPermission => {
      let authUser = await this.authService.getLoggedInUser();
      let user: UserDTO | null = authUser?.users?.find(u => u.server == this.server$.value?.id) ?? null;

      if (build || this.menuItems$.value.length == 0) {
        await this.buildMenu(user, hasPermission);
      }

      if (this.server$.value) {
        this.serverMenu.visible = true;
        this.serverMembers.visible = !!user?.isModerator;
        this.serverAutoRoles.visible = !!user?.isModerator;
        this.serverLevels.visible = !!user?.isModerator;
      } else {
        this.serverMenu.visible = false;
      }

      let menuItems: MenuItem[] = [
        this.dashboard,
        this.serverMenu,
        this.adminMenu
      ];
      this.menuItems$.next(menuItems);
    });
  }
}
